from datetime import datetime

import allure
from pydantic import BaseModel

from overhave.storage import FeatureTypeName


class FeatureInfo(BaseModel):
    """Model for feature info keeping."""

    id: int | None
    name: str | None
    type: FeatureTypeName | None
    tags: list[str] | None
    severity: allure.severity_level | None
    author: str | None
    last_edited_by: str | None
    last_edited_at: datetime | None
    tasks: list[str] | None
    scenarios: str | None


class StrictFeatureInfo(BaseModel):
    """Model for feature info keeping with strict presence of fields."""

    id: int
    name: str
    type: FeatureTypeName
    tags: list[str] = []
    severity: allure.severity_level = allure.severity_level.NORMAL
    author: str
    last_edited_by: str
    last_edited_at: datetime
    tasks: list[str] = []
    scenarios: str



from typing import Any

import httpx
from pydantic import validator

from overhave.transport.http import BaseHttpClientSettings
from overhave.utils import make_url


class TokenizerClientSettings(BaseHttpClientSettings):
    """Important environments and settings for :class:`TokenizerClient`."""

    enabled: bool = False
    url: httpx.URL | None = None  # type: ignore
    initiator: str = "Overhave"
    remote_key: str | None = None
    remote_key_name: str | None = None

    class Config:
        env_prefix = "OVERHAVE_GITLAB_TOKENIZER_"

    @validator("url", pre=True)
    def make_url(cls, v: str | None) -> httpx.URL | None:
        return make_url(v)

    @validator("url", "remote_key", "remote_key_name")
    def validate_remote_key_and_initiator(cls, v: str | None, values: dict[str, Any]) -> str | None:
        if values.get("enabled") and not isinstance(v, str):
            raise ValueError("Please verify that url, remote_key and remote_key_name variables are not nullable!")
        return v
